<?php

namespace Drupal\media_remote_social\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\key\KeyRepositoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Media Remote Social settings.
 */
class MediaRemoteSocialSettingsForm extends ConfigFormBase {

  /**
   * The key repository.
   *
   * @var \Drupal\key\KeyRepositoryInterface
   */
  protected $keyRepository;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a MediaRemoteSocialSettingsForm object.
   *
   * @param \Drupal\key\KeyRepositoryInterface $key_repository
   *   The key repository.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(KeyRepositoryInterface $key_repository, ModuleHandlerInterface $module_handler) {
    $this->keyRepository = $key_repository;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('key.repository'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'media_remote_social_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['media_remote_social.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('media_remote_social.settings');
    
    $form['facebook'] = [
      '#type' => 'details',
      '#title' => $this->t('Meta Settings'),
      '#open' => TRUE,
    ];

    // Get available keys.
    $key_options = [];
    $keys = $this->keyRepository->getKeys();
    foreach ($keys as $key) {
      $key_options[$key->id()] = $key->label();
    }

    $form['facebook']['facebook_access_token_key'] = [
      '#type' => 'select',
      '#title' => $this->t('Meta Access Token'),
      '#description' => $this->t('Select a key that contains your Meta access token for Facebook and Instagram. You can create a new key at <a href="@url">the key management page</a>.', [
        '@url' => '/admin/config/system/keys/add',
      ]),
      '#options' => $key_options,
      '#empty_option' => $this->t('- Select a key -'),
      '#default_value' => $config->get('facebook_access_token_key'),
    ];

    $form['facebook']['help'] = [
      '#type' => 'item',
      '#markup' => $this->t('<strong>How to get a Meta access token:</strong><ol>
        <li>Go to <a href="https://developers.facebook.com/" target="_blank">Meta for Developers</a></li>
        <li>Create a new app or use an existing one</li>
        <li>Add the "Meta oEmbed Read" permission for Facebook and Instagram embedding</li>
        <li>Navigate to Settings > Basic to find your App ID and App Secret</li>
        <li>Generate an access token using the Graph API Explorer or by combining: <code>APP_ID|APP_SECRET</code></li>
        <li>Create a new key in Drupal with this access token value</li>
        <li>Note: For production use, your app must be reviewed and approved by Meta</li>
      </ol>'),
    ];

    $form['legal'] = [
      '#type' => 'details',
      '#title' => $this->t('Legal Documents for Meta App'),
      '#open' => TRUE,
      '#description' => $this->t('Meta requires Privacy Policy, Data Deletion, and Terms of Service URLs when submitting your app for review.'),
    ];

    $module_path = $this->moduleHandler->getModule('media_remote_social')->getPath();
    $form['legal']['documents'] = [
      '#type' => 'item',
      '#markup' => $this->t('<p><strong>Example legal documents are included with this module:</strong></p>
        <ul>
          <li><strong>Privacy Policy:</strong> <code>@privacy</code></li>
          <li><strong>Data Deletion:</strong> <code>@deletion</code></li>
          <li><strong>Terms of Service:</strong> <code>@terms</code></li>
        </ul>
        <p>You can use these documents in several ways:</p>
        <ol>
          <li><strong>Direct file access:</strong> Link to the .md files directly if your server allows it</li>
          <li><strong>Create pages:</strong> Copy the content to create pages on your website</li>
          <li><strong>GitHub hosting:</strong> Host these files in your GitHub repository</li>
        </ol>
        <p><strong>Important:</strong> These are example documents. Review and modify them to match your specific use case and comply with your local regulations.</p>
        <p><a href="@docs_url" target="_blank">View Meta\'s App Review documentation</a></p>', [
        '@privacy' => $module_path . '/privacy-policy.md',
        '@deletion' => $module_path . '/data-deletion.md',
        '@terms' => $module_path . '/terms-of-service.md',
        '@docs_url' => 'https://developers.facebook.com/docs/app-review',
      ]),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('media_remote_social.settings')
      ->set('facebook_access_token_key', $form_state->getValue('facebook_access_token_key'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}