<?php

namespace Drupal\Tests\media_remote_social\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\media\Traits\MediaTypeCreationTrait;
use Drupal\media\Entity\Media;

/**
 * Tests the display of remote social media embeds.
 *
 * @group media_remote_social
 */
class RemoteSocialDisplayTest extends BrowserTestBase {

  use MediaTypeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field',
    'file',
    'image',
    'media',
    'media_remote_social',
    'media_remote_social_test',
    'node',
    'system',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The media storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $mediaStorage;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    
    $this->mediaStorage = $this->container->get('entity_type.manager')->getStorage('media');
    
    // Create a user with permissions to create and view media.
    $account = $this->drupalCreateUser([
      'create media',
      'view media',
    ]);
    $this->drupalLogin($account);
  }

  /**
   * Data provider for social media display tests.
   *
   * @return array[]
   *   The test cases.
   */
  public static function providerSocialMediaDisplay(): array {
    return [
      'Facebook post' => [
        'https://www.facebook.com/20531316728/posts/10154009990506729/',
        'facebook',
        'fb-post',
        'connect.facebook.net',
      ],
      'Instagram post' => [
        'https://www.instagram.com/p/ABC123DEF456/',
        'instagram',
        'instagram-media',
        'instagram.com/embed.js',
      ],
    ];
  }

  /**
   * Tests that social media URLs are properly displayed.
   *
   * @param string $url
   *   The social media URL to test.
   * @param string $provider
   *   The provider name (used for fixture file).
   * @param string $embed_class
   *   A class or attribute that should be present in the embed HTML.
   * @param string $script_src
   *   Part of the script source that should be present.
   *
   * @dataProvider providerSocialMediaDisplay
   */
  public function testSocialMediaDisplay(string $url, string $provider, string $embed_class, string $script_src): void {
    // Set up the test middleware to return our fixture data.
    $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
    $fixture_file = __DIR__ . '/../../fixtures/' . $provider . '.json';
    
    $state = $this->container->get('state');
    $state->set('Drupal\media_remote_social_test\ResourceMiddleware', [
      $resource_url => $fixture_file,
    ]);

    // Create a media entity with the social URL.
    $media = Media::create([
      'bundle' => 'remote_social',
      'name' => 'Test ' . ucfirst($provider) . ' Media',
      'field_media_oembed_social' => $url,
    ]);
    $media->save();
    
    // Visit the media canonical page.
    $this->drupalGet($media->toUrl());
    
    // Assert that we get a successful response.
    $this->assertSession()->statusCodeEquals(200);
    
    // Assert that the media title is displayed.
    $this->assertSession()->pageTextContains('Test ' . ucfirst($provider) . ' Media');
    
    // Assert that the embed HTML is rendered.
    $this->assertSession()->responseContains($embed_class);
    $this->assertSession()->responseContains($script_src);
    
    // For providers that use iframes or specific embed patterns, check for them.
    switch ($provider) {
      case 'facebook':
        $this->assertSession()->responseContains('data-href="' . $url . '"');
        break;
        
      case 'instagram':
        $this->assertSession()->responseContains('data-instgrm-permalink');
        break;
    }
  }

  /**
   * Tests that the media type configuration is correct.
   */
  public function testMediaTypeConfiguration(): void {
    // Load the remote_social media type.
    $media_type = $this->container->get('entity_type.manager')
      ->getStorage('media_type')
      ->load('remote_social');
    
    $this->assertNotNull($media_type, 'Remote social media type exists.');
    $this->assertEquals('Remote Social', $media_type->label());
    $this->assertEquals('oembed:social', $media_type->getSource()->getPluginId());
    
    // Check that the source field exists.
    $source_field = $media_type->getSource()->getSourceFieldDefinition($media_type);
    $this->assertEquals('field_media_oembed_social', $source_field->getName());
    $this->assertEquals('string', $source_field->getType());
  }

  /**
   * Tests creating media through the UI.
   */
  public function testMediaCreationUI(): void {
    $this->drupalGet('media/add/remote_social');
    $this->assertSession()->statusCodeEquals(200);
    
    // Check that the form has the correct field.
    $this->assertSession()->fieldExists('field_media_oembed_social[0][value]');
    
    // Test with a Facebook URL.
    $url = 'https://www.facebook.com/20531316728/posts/10154009990506729/';
    $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
    
    $state = $this->container->get('state');
    $state->set('Drupal\media_remote_social_test\ResourceMiddleware', [
      $resource_url => __DIR__ . '/../../fixtures/facebook.json',
    ]);
    
    // Submit the form.
    $edit = [
      'name[0][value]' => 'Test Facebook Post',
      'field_media_oembed_social[0][value]' => $url,
    ];
    $this->submitForm($edit, 'Save');
    
    // Check that the media was created successfully.
    $this->assertSession()->pageTextContains('Remote Social Test Facebook Post has been created.');
    
    // Verify the media entity was saved correctly.
    $media_entities = $this->mediaStorage->loadByProperties(['name' => 'Test Facebook Post']);
    $this->assertCount(1, $media_entities, 'Media entity was created.');
    
    $media = reset($media_entities);
    $this->assertEquals($url, $media->get('field_media_oembed_social')->value);
  }
}